import { createWriteStream, mkdirSync, existsSync, unlinkSync } from 'fs';
import { join, dirname } from 'path';
import { fileURLToPath } from 'url';
import { Readable } from 'stream';
import { randomUUID } from 'crypto';

const __filename = fileURLToPath(import.meta.url);
const __dirname = dirname(__filename);

// Configure upload directory
const UPLOAD_DIR = join(__dirname, '..', '..', 'uploads');

// Ensure the upload directory exists
if (!existsSync(UPLOAD_DIR)) {
  mkdirSync(UPLOAD_DIR, { recursive: true });
}

// Define allowed file types
const ALLOWED_MIME_TYPES = [
  'image/jpeg',
  'image/png',
  'image/gif',
  'image/webp'
];

// Maximum file size: 5MB
const MAX_FILE_SIZE = 5 * 1024 * 1024;

// Helper function to check if a file is valid
export const isValidFile = (mimetype: string, size: number): boolean => {
  return ALLOWED_MIME_TYPES.includes(mimetype) && size <= MAX_FILE_SIZE;
};

// Helper function to save a file to disk
export const saveFile = (
  buffer: Buffer, 
  mimetype: string
): Promise<string> => {
  return new Promise((resolve, reject) => {
    const fileExtension = mimetype.split('/')[1];
    const fileName = `${randomUUID()}.${fileExtension}`;
    const filePath = join(UPLOAD_DIR, fileName);
    
    const stream = new Readable();
    stream.push(buffer);
    stream.push(null);
    
    const writeStream = createWriteStream(filePath);
    
    stream.pipe(writeStream);
    
    writeStream.on('finish', () => {
      resolve(fileName);
    });
    
    writeStream.on('error', (err) => {
      reject(err);
    });
  });
};

// Helper function to delete a file
export const deleteFile = (fileName: string): boolean => {
  try {
    const filePath = join(UPLOAD_DIR, fileName);
    if (existsSync(filePath)) {
      unlinkSync(filePath);
      return true;
    }
    return false;
  } catch (error) {
    console.error('Error deleting file:', error);
    return false;
  }
};

// Helper function to get the public URL for a file
export const getPublicUrl = (fileName: string): string => {
  return `/uploads/${fileName}`;
};
